package dj15.server;


/**
 * ControllServlet
 * @author Hiromasa Ogawa
 * @version 1.0
 */
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import dj15.server.command.*;
import dj15.server.logger.*;

public class ControllServlet extends HttpServlet {
    public ControllServlet() {
        super();

    }

    /**
     * HTTP GET
     */
    public void doGet (
        HttpServletRequest request,
        HttpServletResponse response
		) throws ServletException,IOException {

        performTask(request, response);
    }

    /**
     * HTTP POST
     */
    public void doPost (
        HttpServletRequest request,
        HttpServletResponse response
		) throws ServletException,IOException {

        performTask(request, response);
    }

    /**
     * return Servlet Info
     */
    public String getServletInfo() {
        return super.getServletInfo();
    }

	/**
	 * Servlet Initialization
	 * @see javax.servlet.GenericServlet#init()
	 */
    public void init() {
    	ServletConfig config = this.getServletConfig();
    	String loggerLevel = config.getInitParameter("logger");
		Level priority = Level.getIgnoreEqualLoggingLevel(loggerLevel);
		PersistenceLogger.createLogger(this.getServletContext(),priority);
    }

    /**
     * select and call business logic
     */
    public void performTask (
         HttpServletRequest request,
         HttpServletResponse response
    ) throws ServletException,IOException{
		ServletContext context = getServletContext();
        String page;
        PersistenceLogger logger = PersistenceLogger.getLogger();
		//Logging
//		Logger logger = Logger.getLogger(this.getClass().getName());
//		logger.info("URI    : " + request.getRequestURI());	//URI
//		logger.info("Query  : " + request.getQueryString());//QueryString
//		logger.info("METHOD : " + request.getMethod());		//METHOD
/*
		context.log("[" + this.getClass().getName() + "]");	//Class Name
		context.log("URI    : " + request.getRequestURI());		//URI
		context.log("Query  : " + request.getQueryString());	//QueryString
		context.log("METHOD : " + request.getMethod());			//METHOD
*/
		logger.log("[" + this.getClass().getName() + "]",Level.INFO);	//Class Name
		logger.log("URI    : " + request.getRequestURI(),Level.INFO);	//URI
		logger.log("Query  : " + request.getQueryString(),Level.INFO);	//QueryString
		logger.log("METHOD : " + request.getMethod(),Level.INFO);		//METHOD

        try {
            RequestHelper requestHelper
                = new RequestHelper(request, response,getServletContext());
            Command command = requestHelper.getCommand();
			if (command != null) {
				command.execute(requestHelper);
			}

        } catch (Throwable th) {
			getServletContext().log("Server Error",th);
            response.sendError(response.SC_INTERNAL_SERVER_ERROR);
        }
    }

}