package dj15.server;

/**
 * RequsetHelper
 * @author Hiromasa Ogawa
 * @version 1.0
 */
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.*;
import dj15.server.command.*;
import dj15.server.logger.*;

public class RequestHelper {
    HttpServletRequest request;
    HttpServletResponse response;
    ServletContext context;
	String actionName;
	protected PersistenceLogger logger = PersistenceLogger.getLogger();

    public RequestHelper(
        HttpServletRequest req,
        HttpServletResponse resp,
        ServletContext ctxt) {

        super();

        request = req;
        response = resp;
        context = ctxt;
    }

    public Command getCommand() throws Exception{

		String uri = request.getRequestURI();
		int sidx = uri.lastIndexOf('/');
		int eidx = uri.lastIndexOf(".dj15");
		actionName = uri.substring(sidx+1,eidx);
		ResourceBundle rb = ResourceBundle.getBundle("actionmap");

		try {
			String className = rb.getString(actionName);
			return (Command) Class.forName(className).newInstance();
		} catch (MissingResourceException mre) {
//			context.log("BAD REQUEST > no registered action:"+actionName);
			logger.log("BAD REQUEST > no registered action:"+actionName, Level.ERROR);
			response.sendError(HttpServletResponse.SC_BAD_REQUEST);
			return null;
		}
    }

    public HttpServletRequest getRequest() {
        return request;
    }

    public HttpServletResponse getResponse() {
        return response;
    }

    public ServletContext getContext() {
        return context;
    }

	public String getActionName() {
		return actionName;
	}

	public void retBadRequest(String param, String value) throws ServletException,IOException{
//		context.log("BAD REQUEST["+actionName+"] > "+ param + ":" + value);
		logger.log("BAD REQUEST["+actionName+"] > "+ param + ":" + value , Level.ERROR);
		response.sendError(HttpServletResponse.SC_BAD_REQUEST);
	}

	public void log(String msg){
		context.log(msg);
	}

	public void log(String msg,Throwable th){
		context.log(msg, th);
	}
}