package dj15.server.command;

/**
 * GetFileDataCommand
 * @author Hiromasa Ogawa
 * @version 1.0
 */
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import dj15.server.*;
import dj15.server.util.*;
import dj15.server.logger.*;

public class GetFileDataCommand implements Command{
	protected static final PersistenceLogger logger = PersistenceLogger.getLogger();	

    public GetFileDataCommand() {
        super();
    }

    public void execute( RequestHelper requestHelper)
                        throws ServletException,IOException {

        String fnam = requestHelper.getRequest().getParameter("name");
        if(fnam == null) {
			requestHelper.retBadRequest("name",fnam);
        }
        File file = new File(requestHelper.getContext().getRealPath(fnam));
        if(!file.exists()){
			requestHelper.retBadRequest("name",fnam);
        }

        String sofs = requestHelper.getRequest().getParameter("ofs");
        if(sofs == null) {
			requestHelper.retBadRequest("ofs",sofs);
        }
        int ofs = 0;
        try {
            ofs = Integer.parseInt(sofs);
        } catch (NumberFormatException nfe) {
			requestHelper.retBadRequest("ofs",sofs);
        }

        String ssize = requestHelper.getRequest().getParameter("sz");
        if(ssize == null) {
			requestHelper.retBadRequest("sz",ssize);
        }
        int size = 0;
        try {
            size = Integer.parseInt(ssize);
        } catch (NumberFormatException nfe) {}
        if (size == 0) {
            requestHelper.retBadRequest("sz",ssize);
        }

        /** create response byte[] */
        byte[] buf = new byte[size];
        long flen;
        RandomAccessFile raf = new RandomAccessFile(file,"r");
        flen = raf.length();
        raf.seek((long)ofs);
        raf.read(buf);
        raf.close();

		//Logging
		/*
		requestHelper.getContext().log("[" + this.getClass().getName() + "]");//Class Name
		requestHelper.getContext().log("filename : " + fnam);				//File Name
		requestHelper.getContext().log("offset   : " + ofs);				//Download Offset
		requestHelper.getContext().log("size     : " + size);				//Download Size
		requestHelper.getContext().log("totalsize: " + flen);		//Download TotalSize
		*/
		logger.log("[" + this.getClass().getName() + "]", Level.DEBUG);//Class Name
		logger.log("filename : " + fnam, Level.DEBUG);				//File Name
		logger.log("offset   : " + ofs, Level.DEBUG);					//Download Offset
		logger.log("size     : " + size, Level.DEBUG);				//Download Size
		logger.log("totalsize: " + flen, Level.DEBUG);				//Download TotalSize
        
        requestHelper.getResponse().setContentType("application/octet-stream");
        DataOutputStream out = new DataOutputStream(
                                    requestHelper.getResponse().getOutputStream());
        out.write(buf);
        out.close();
        requestHelper.getResponse().setContentLength(buf.length);
    }
}