	package dj15.server.command.local;

/**
 * LocalBaseCommand
 * @author Hiromasa Ogawa
 * @version 1.0
 */

import dj15.server.command.Command;
import dj15.server.RequestHelper;
import dj15.server.logger.*;
import java.io.*;
import javax.servlet.ServletException;


abstract class LocalBaseCommand implements Command {
	protected static final PersistenceLogger logger = PersistenceLogger.getLogger();
	
	public String readParamString(RequestHelper rh)
			throws ServletException,IOException {
		String str = rh.getRequest().getParameter("str");
		str = new String(str.getBytes("ISO8859_1"),"UTF8");
		return str;
	}

	public String readUTF(RequestHelper rh)
			throws ServletException,IOException {
		DataInputStream dis = new DataInputStream(rh.getRequest().getInputStream());
		String ret = dis.readUTF();
		dis.close();

		return ret;
	}

    public String readByteString(RequestHelper rh)
            throws ServletException,IOException {
        //rh.log("ContentLength:"+rh.getRequest().getContentLength());
        DataInputStream dis = new DataInputStream(rh.getRequest().getInputStream());
        byte[] b = new byte[rh.getRequest().getContentLength()];
        dis.readFully(b);
        String ret = new String(b,"UTF8");
        /******
        ByteArrayOutputStream bao = new ByteArrayOutputStream();
        byte[] b = new byte[128];
        int sz;
        while ((sz = dis.read(b)) != -1) {
            bao.write(b,0,sz);
        }
        String ret = new String(bao.toByteArray(),"UTF8");
        bao.close();
        *******/

        dis.close();
        return ret;
    }
	public void writeUTF(RequestHelper rh, String str)
			throws ServletException,IOException {
		rh.getResponse().setContentType("application/octet-stream");
		ByteArrayOutputStream bo = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(bo);

		dos.writeUTF(str);

		rh.getResponse().setContentLength(bo.size());
		bo.writeTo(rh.getResponse().getOutputStream());
		dos.close();
		bo.close();

		//Logging
		/*
		rh.getContext().log("[" + this.getClass().getName() + "]");//Class Name
		rh.getContext().log("String(UTF-8) : " + str);				//Request String(UTF-8)
		rh.getContext().log("String(hex)   : " + toHexString(str));	//Request String(Hex)
		*/
		logger.log("[" + this.getClass().getName() + "]", Level.DEBUG);	//Class Name
		logger.log("String(EUC-JP) : " + (new String(str.getBytes("EUC_JP"))), Level.DEBUG);			//Request String(EUC_JP)			
		logger.log("String(UTF-8) : " + (new String(str.getBytes("UTF-8"))), Level.DEBUG);				//Request String(UTF-8)			
		logger.log("String(hex)   : " + toHexString(str), Level.DEBUG);	//Request String(Hex)
	}

	public void writeBytes(RequestHelper rh, String str)
			throws ServletException,IOException {
		rh.getResponse().setContentType("application/octet-stream");
		ByteArrayOutputStream bo = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(bo);

		dos.write(str.getBytes("UTF8"));

		rh.getResponse().setContentLength(bo.size());
		bo.writeTo(rh.getResponse().getOutputStream());
		dos.close();
		bo.close();

		//Logging
		/*
		rh.getContext().log("[" + this.getClass().getName() + "]");//Class Name	
		rh.getContext().log("String(UTF-8) : " + str);				//Request String(UTF-8)
		rh.getContext().log("String(hex)   : " + toHexString(str));	//Request String(UTF-8)
		*/
		logger.log("[" + this.getClass().getName() + "]", Level.DEBUG);//Class Name
		logger.log("String(EUC-JP) : " + (new String(str.getBytes("EUC_JP"))), Level.DEBUG);			//Request String(EUC_JP)			
		logger.log("String(UTF-8) : " + (new String(str.getBytes("UTF-8"))), Level.DEBUG);				//Request String(UTF-8)			
		logger.log("String(hex)   : " + toHexString(str), Level.DEBUG);	//Request String(Hex)
	}
	
	private String toHexString(String str){
		StringBuffer result = new StringBuffer();
		if(str == null)	return "";
		for(int i = 0 ; i < str.length() ; i++){
			char ch = str.charAt(i);
			String hex = Integer.toHexString((int)ch);
			result.append(hex + " ");
		}		
		
		return result.toString();
	}
}