package dj15.server.command.worldwatch;

/**
 * GetScratchDataCommand
 * @author Hiromasa Ogawa
 * @version 1.0
 */
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import dj15.server.*;
import dj15.server.command.*;
import dj15.server.util.*;
import dj15.server.logger.*;

public class GetScratchDataCommand implements Command {
	private static final String packagePrefix = "dj15.server.command.worldwatch.";
	private static final String TMP_DIR = "/tmp/dj15/worldwatch/";
	protected static final PersistenceLogger logger = PersistenceLogger.getLogger();
	RequestHelper requestHelper;
	

    public void execute(RequestHelper requestHelper) throws ServletException, IOException {
        this.requestHelper = requestHelper;

		String fnam = requestHelper.getRequest().getParameter("fname");
        if(fnam == null) {
            requestHelper.retBadRequest("fname",fnam);
            return;
        }
        //File file = new File(requestHelper.getContext().getRealPath("/worldwatch/"+fnam+".scr"));
		File file = new File(TMP_DIR+fnam+"_"+Util.getLocale().getLanguage()+"_"+Util.getLocale().getCountry()+".scr");
		if(!file.exists()){
            try {
                createScratchData(fnam);
            } catch (Exception e) {
				//requestHelper.log("createScratchData failed.",e);
				logger.log("createScratchData failed.",Level.ERROR);
                requestHelper.getResponse().sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
                return;
            }
        }

        String sofs = requestHelper.getRequest().getParameter("pos");
        if(sofs == null) {
			requestHelper.retBadRequest("pos",sofs);
			return;
        }
        int ofs = 0;
        try {
            ofs = Integer.parseInt(sofs);
        } catch (NumberFormatException nfe) {
			requestHelper.retBadRequest("pos",sofs);
			return;
        }

        /** fix 10kbytes */
        int size = 10240;

        /** calculate offset position */
        ofs = ofs * size;

        /** create response byte[] */
        byte[] buf = new byte[size];
        long flen;
        RandomAccessFile raf = new RandomAccessFile(file,"r");
        flen = raf.length();
        raf.seek((long)ofs);
        raf.read(buf);
        raf.close();

		//Logging
		/*
		requestHelper.getContext().log("[" + this.getClass().getName() + "]");//Class Name
		requestHelper.getContext().log("Locale   : " + Util.getLocale().getLanguage()+"_"+Util.getLocale().getCountry());
																			//Locale
		requestHelper.getContext().log("filename : " + fnam);				//File Name
		requestHelper.getContext().log("offset   : " + ofs);				//Download Offset
		requestHelper.getContext().log("size     : " + size);				//Download Size
		requestHelper.getContext().log("totalsize: " + flen);				//Download TotalSize
		*/
		logger.log("[" + this.getClass().getName() + "]", Level.DEBUG);	//Class Name
		logger.log("Locale   : " + Util.getLocale().getLanguage()+"_"+Util.getLocale().getCountry(), Level.DEBUG);
																		//Locale
		logger.log("filename : " + fnam, Level.DEBUG);					//File Name
		logger.log("offset   : " + ofs, Level.DEBUG);					//Download Offset
		logger.log("size     : " + size, Level.DEBUG);					//Download Size
		logger.log("totalsize: " + flen, Level.DEBUG);					//Download TotalSize


        requestHelper.getResponse().setContentType("application/octet-stream");
        DataOutputStream out = new DataOutputStream(
                                    requestHelper.getResponse().getOutputStream());
        out.write(buf);
        out.close();
        requestHelper.getResponse().setContentLength(buf.length);
    }

    private void createScratchData(String fnam) throws IOException,FileNotFoundException,Exception{
        byte[] tz = makeTimeZoneData(packagePrefix + fnam + "_tz");
        byte[] flg = makeImgData(packagePrefix+fnam + "_flg",1536);
        byte[] parts = makeImgData(packagePrefix+fnam + "_parts",1024,true);

		byte[] b = new byte[102400];
        ByteArrayOutputStream bo = new ByteArrayOutputStream();
        DataOutputStream bod = new DataOutputStream(bo);

		/* system infomation */
        bod.writeByte(1);             // first start flag
        bod.writeByte(tz[0]);     // number of time zone
		bod.writeInt(32);            // time zone offset
		bod.writeByte(flg[0]);    // number of national flag image
		bod.writeInt(10240);         // nationalflag image offset
		bod.writeByte(parts[0]);  // number of screen parts 
		bod.writeInt(81920);         // screen parts offset
		System.arraycopy(bo.toByteArray(),0,b,0,bo.size());
		bod.close();
        bo.close();

        /* time zones */
		System.arraycopy(tz,1,b,32,tz.length-1);

        /* national flag images */
		System.arraycopy(flg,1,b,10240,flg.length-1);

        /* screen parts */
		System.arraycopy(parts,1,b,81920,parts.length-1);

		/*
		 * output to file
		 */
		//FileOutputStream fo =  new FileOutputStream(requestHelper.getContext().getRealPath("/worldwatch/"+fnam+".scr"));
		new File(TMP_DIR).mkdirs();
		File f = new File(TMP_DIR+fnam+"_"+Util.getLocale().getLanguage()+"_"+Util.getLocale().getCountry()+".scr");
		FileOutputStream fo =  new FileOutputStream(f);

		fo.write(b);
		fo.close();
	//	f.deleteOnExit();
    }

    private byte[] makeTimeZoneData(String pfnam) throws IOException, Exception{
		ResourceBundle rb = ResourceBundle.getBundle(pfnam, Util.getLocale());
		int len = Integer.parseInt(rb.getString("info_num"));
		ByteArrayOutputStream bo = new ByteArrayOutputStream();
		bo.write(len);
		for(int i=0; i<len; i++ ) {
			String s = rb.getString("info_"+i);
			String[] split = new String[5];
			int j = 0;
			StringTokenizer st = new StringTokenizer(s,",");
			while(st.hasMoreTokens()){
				split[j++] = st.nextToken(",");
			}

			byte[] b = new byte[100];
			byte [] tmp = split[0].getBytes("UTF8"); // country name
			System.arraycopy(tmp,0,b,0,tmp.length);
			tmp = split[1].getBytes("UTF8");         // area name
			System.arraycopy(tmp,0,b,48,tmp.length);
			b[96] = Byte.parseByte(split[2]);        // national flag index
			b[97] = Byte.parseByte(split[3]);        // time difference against GMT x10
			b[98] = Byte.parseByte(split[4]);        // summer time flag
			bo.write(b);
		}

		byte[] ret = bo.toByteArray();
		bo.close();

		return ret;
    }

	private byte[] makeImgData(String pfnam,int ds) throws IOException, FileNotFoundException{
		return makeImgData(pfnam, ds, false);
    }

	private byte[] makeImgData(String pfnam,int ds, boolean esp)
			throws IOException, FileNotFoundException{
		ResourceBundle rb = ResourceBundle.getBundle(pfnam);
		int len = Integer.parseInt(rb.getString("img_num"));
		ByteArrayOutputStream bo = new ByteArrayOutputStream();
		bo.write(len);
		for(int i=0; i<len; i++ ) {
			String s = rb.getString("img_"+i);
			File file = new File(requestHelper.getContext().getRealPath("/worldwatch/img/"+s));

			byte b[];
			if(esp == true && i==len-1) {
				b = new byte[3072];
			} else {
				b = new byte[ds];
			}
			RandomAccessFile raf = new RandomAccessFile(file,"r");
			raf.read(b);
			raf.close();

			bo.write(b);
		}
		byte[] ret = bo.toByteArray();
		bo.close();

		return ret;
	}

	public static void delImgData(String fnam){
		new File(TMP_DIR+fnam+"_"+Util.getLocale().getLanguage()+"_"+Util.getLocale().getCountry()+".scr").delete();
	}
}
