package dj15.server.logger;

/**
 * @author user
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
import javax.servlet.ServletContext;
import java.io.*;

public class PersistenceLogger {
	private static PersistenceLogger singleton = new PersistenceLogger();
	private static ServletContext currentContext = null;
	private static Level currentLevel = null;


	/**
	 * Constructor
	 */
	private PersistenceLogger(){}

	
	/**
	 * get PersistenceLogger class instance
	 * @return PersistenceLogger
	 */
	private static PersistenceLogger getInstance(){
		return singleton;	
	}
	
	/**
	 * create logger
	 * @param context				ServletContext
	 * @return PersistenceLogger
	 */
	public static PersistenceLogger createLogger(ServletContext context){
		return createLogger(context, Level.getDefault());
	}
	
	/**
	 * create logger
	 * @param context				ServletContext
	 * @param level				Logging Level
	 * @return PersistenceLogger
	 */
	public static PersistenceLogger createLogger(ServletContext context, Level level){
		PersistenceLogger my = PersistenceLogger.getInstance();
		my.currentContext = context;
		my.currentLevel   = level;
		
		return my;
	}
	
	/**
	 * get logger
	 * @return PersistenceLogger
	 */
	public static PersistenceLogger getLogger(){
		PersistenceLogger my = PersistenceLogger.getInstance();
		if(my.currentContext == null || my.currentLevel == null){
			return createLogger(null, Level.getDefault());
		}else{
			return my;	
		}	
	}
	
	/**
	 * log output
	 * @param msg		output message
	 * @param level	logging level
	 */
	public void log(String msg, Level level){
		if(level == null){
			level = Level.getDefault();	
		}
		String levelStr = "[" + level.toString() + "]";
		
		if(this.currentLevel.isLoggingLevel(level)){
			if(this.currentContext == null){	//stdout
				System.out.println(levelStr + msg);				
			}else{								//ServletContext#log
				this.currentContext.log(levelStr + msg);	
			}
		}
	}
	
	/** for debug **/
	public static void main(String[] args){
		PersistenceLogger logger = PersistenceLogger.createLogger(null, Level.INFO);
		
		logger.log("DEBUG LEVEL", Level.DEBUG);
		logger.log("INFO LEVEL" , Level.INFO);
		logger.log("ERROR LEVEL", Level.ERROR);
	}	
}
